/**
 * @file
 *
 * Generate a working set with a random access pattern.
 *
 * @author Jens Teubner <jens.teubner@inf.ethz.ch>
 *
 * (c) 2010 ETH Zurich, Systems Group
 *
 * $Id: random.c 1100 2011-08-17 14:32:58Z jteubner $
 */

#include "config.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "pattern/pattern.h"

/**
 * Fill the working set @a elems such that scanning it will lead to
 * a random memory access pattern (each element points to its
 * follower; the last element points back to the first).
 *
 * @param elems  working set allocated using generate_workset();
 *               starting from @a elems[0], @c next pointers will be
 *               filled by this function
 * @param size   size of array (in bytes) specified by @a elems
 * @param stride stride size; must be a positive multiple of the
 *               machine pointer size
 *
 * @return pointer to @a elems
 */
elem_t *
pattern_rand (elem_t *elems, size_t size, unsigned int stride)
{
    unsigned long  nstride = stride / sizeof (elem_t);
    unsigned long  nitems = size / stride;
    unsigned long *items = malloc (nitems * sizeof (*items));

    assert (items);

    /*
     * fill array items[] with a random permutation of numbers
     * [0..nitems]
     */
    for (unsigned long i = 0; i < nitems; i++)
        items[i] = i;

    for (unsigned long i = nitems-1; i > 0; i--)
    {
        unsigned long k = (unsigned long long) i * random () / RAND_MAX;
        unsigned long t;

        t = items[i];
        items[i] = items[k];
        items[k] = t;
    }

    /* fill the working set with pointers, driven by the permuted sequence */
    for (unsigned int i = 0; i < nitems-1; i++)
        elems[items[i] * nstride].next = &(elems[items[i+1] * nstride]);

    elems[items[nitems-1] * nstride].next = &(elems[items[0] * nstride]);

    free (items);

    return elems;
}
