/**
 * @file
 *
 * Data structures to describe a particular experimental setup.
 *
 * @author Jens Teubner <jens.teubner@inf.ethz.ch>
 *
 * (c) 2010 ETH Zurich, Systems Group
 *
 * $Id: setup.h 1100 2011-08-17 14:32:58Z jteubner $
 */

#include "config.h"

#ifndef SETUP_H
#define SETUP_H

#include <pthread.h>

#ifdef HAVE_LIBNUMA
#include <numa.h>
#endif

#include "workset/workset.h"

struct intlist_t {
    unsigned int    count;
    unsigned long  *values;
};
typedef struct intlist_t intlist_t;

struct parameter_t {
    char          *name;
    unsigned long  curr_value;
    intlist_t      values;
};
typedef struct parameter_t parameter_t;

struct parameters_t {
    unsigned int   count;
    parameter_t  **params;
};
typedef struct parameters_t parameters_t;


enum access_pattern_t { pat_sequential, pat_random };
typedef enum access_pattern_t access_pattern_t;

#include "workset/accessmode.h"

struct memory_region_t {

    /**
     * Identifier for this memory region, as stated with the @c id
     * field in the XML configuration file.
     */
    char             *id;

    /**
     * List of NUMA nodes where this memory region may be placed.
     */
    char             *numa_nodes;

#ifdef HAVE_LIBNUMA
    struct bitmask   *numa_nodes_mask;
#else
    void             *numa_nodes_mask;
#endif

    unsigned long    *size;
    unsigned long    *stride;

    access_pattern_t  pattern;

    /* ---- these will be populated at runtime ---- */

    /** allocated working set in this memory regions; otherwise @c NULL */
    elem_t           *workset;

    /**
     * size of memory region allocated at @a workset; this is because
     * the NUMA function numa_free() asks for this information
     */
    size_t            ws_size;
};
typedef struct memory_region_t memory_region_t;

struct memory_regions_t {
    unsigned int       count;
    memory_region_t  **regions;
};
typedef struct memory_regions_t memory_regions_t;

struct thread_t {

    /* --- these will be populated when setup is parsed --- */
    char             *id;
    int               numa_node;
    memory_region_t  *region;

    /** access mode (read-only or read/write) */
    accessmode_t      mode;

    /* --- these will be populated when thread is spawned --- */
    pthread_t         thread;
    unsigned int      num_runs;
    double           *nsec_per_step;

};
typedef struct thread_t thread_t;

struct threads_t {
    unsigned int   count;
    thread_t     **threads;
};
typedef struct threads_t threads_t;

struct setup_t {

    parameters_t      parameters;

    memory_regions_t  regions;

    threads_t         threads;

};
typedef struct setup_t setup_t;

void parse_setup_file (setup_t *setup, const char *filename);

#endif  /* SETUP_H? */
